(module angel-sequence (lib "slideshow.ss" "slideshow")
  (require (lib "mred.ss" "mred")
           (lib "class.ss")
           (lib "etc.ss")
           (lib "math.ss")
           (lib "step.ss" "slideshow"))
  
  (provide angel-slides simpler-angel-slides
           scheme-angel-file
           scheme-angel-meta-file
           devil-machine
           together-arrows
           dircomm
           mred-machine
           mred-logo)
  
  (define (halo w h)
    (dc (lambda (dc x y)
	  (let ([p (send dc get-pen)]
		[b (send dc get-brush)])
	    (send dc set-pen (find-pen "yellow" 4))
	    (send dc set-brush (find-brush "white" 'transparent))
	    (send dc draw-ellipse (+ x 2) (+ y 2) (- w 4) (- h 4))
	    (send dc set-brush b)
	    (send dc set-pen p)))
	w h 0 0))

  (define (mk-scheme-angel-file lambda?)
    (ht-append (angel-wing 55 80 #t)
	       (vc-append
		10
		(halo 75 25)
		(cc-superimpose
		 (file-icon 80 115 #t #f)
		 (if lambda? 
		     (colorize (text "l" 'symbol 48) "blue")
		     (blank))))
	       (angel-wing 55 80 #f)))

  (define scheme-angel-file (mk-scheme-angel-file #t))

  (define scheme-angel-meta-file
    (let ([files (inset
		  (vl-append
		   scheme-angel-file
		   (inset scheme-angel-file font-size (/ font-size 2) 0 0))
		  (/ font-size 3))])
      (cb-superimpose
       (scale (mk-scheme-angel-file #f) 3.2 3)
       files)))
  
  (define mred-logo (bitmap (build-path (this-expression-source-directory) "plt-med-shield.gif")))
  (define win-logo (bitmap (build-path (this-expression-source-directory) "windows.gif")))

  (define machine-size 1.5)

  (define devil-machine (desktop-machine machine-size '(devil)))
  (define mred-machine (desktop-machine machine-size '(plt)))

  (define down-arrow (colorize (arrow 30 (- (/ pi 2))) "green"))
  (define downright-arrow (colorize (arrow 30 (- (/ pi 4))) "green"))
  (define downleft-arrow (colorize (arrow 30 (- (* pi 3/4))) "green"))

  (define ->pt (lambda (l)
		 (map (lambda (p)
			(make-object point% (car p) (cadr p)))
		      l)))

  (define (mk-bincomm s)
    (dc (lambda (dc x y)
	  (let-values ([(sx sy) (send dc get-scale)]
		       [(dx dy) (send dc get-origin)]
		       [(op) (send dc get-pen)]
		       [(ob) (send dc get-brush)])
	    (send dc set-origin (+ dx (* sx x)) (+ dy (* sy y)))
	    (send dc set-scale (* sx s) (* sy s))

	    (send dc set-brush
		  (send the-brush-list
			find-or-create-brush
			"yellow"
			'solid))
	    (send dc set-pen
		  (send the-pen-list
			find-or-create-pen
			"black"
			0
			'transparent))

	    (send dc draw-polygon
		  (->pt '((0 10)
			  (10 0)
			  (20 10)))
		  0 0)

	    (send dc draw-polygon
		  (->pt '((0 10)
			  (10 0)
			  (20 10)))
		  100 0)

	    (let ([r (make-object region% dc)]
		  [r2 (make-object region% dc)])
	      (send r set-ellipse 5 -50 110 110)
	      (send r2 set-ellipse 15 -40 90 90)
	      (send r subtract r2)
	      (send dc set-clipping-region r)
	      (send dc draw-rectangle 5 10 110 110)
	      (send dc set-clipping-region #f))
	    
	    (let ([s "1010101010"])
	      (send dc set-font (send the-font-list find-or-create-font
				      8 'modern 'normal 'normal))
	      (let-values ([(w h a d) (send dc get-text-extent s)])
		(send dc draw-text s (/ (- 120 w) 2) (- 65 h 4))))

	    (send dc set-pen op)
	    (send dc set-brush ob)

	    (send dc set-origin dx dy)
	    (send dc set-scale sx sy)))
	(* s 120) (* s 65) 0 0))

  (define (mk-dircomm s)
    (dc (lambda (dc x y)
	  (let-values ([(sx sy) (send dc get-scale)]
		       [(dx dy) (send dc get-origin)]
		       [(op) (send dc get-pen)]
		       [(ob) (send dc get-brush)])
	    (send dc set-origin (+ dx (* sx x)) (+ dy (* sy y)))
	    (send dc set-scale (* sx s) (* sy s))

	    (send dc set-brush
		  (send the-brush-list
			find-or-create-brush
			"yellow"
			'solid))
	    (send dc set-pen
		  (send the-pen-list
			find-or-create-pen
			"black"
			0
			'transparent))

	    (send dc draw-polygon
		  (->pt '((10 0)
			  (0 10)
			  (10 20)))
		  0 0)
	    (send dc draw-polygon
		  (->pt '((0 0)
			  (10 10)
			  (0 20)))
		  85)

	    (send dc draw-rectangle
		  5 5 80 10)

	    (let ([s "(  (x) ...)"]
		  [greek (send the-font-list find-or-create-font
			       8 'symbol 'normal 'normal)]
		  [plain (send the-font-list find-or-create-font
			       6 'modern 'normal 'normal)])
	      (send dc set-font plain)
	      (let*-values ([(w h d a) (send dc get-text-extent s)]
			    [(lw lh ld la) (send dc get-text-extent "l" greek)]
			    [(maxa) (max (- h d) (- lh ld))]
			    [(ss) (/ (- 20 (max h lh)) 2)])
		(send dc draw-text s (/ (- 90 w) 2) (+ ss (- maxa (- h d))))
		(send dc set-font greek)
		(send dc draw-text "l" (+ (/ (- 90 w) 2) lw) (+ ss (- maxa (- lh ld))))))

	    (send dc set-pen op)
	    (send dc set-brush ob)
	    (send dc set-origin dx dy)
	    (send dc set-scale sx sy)))
	(* s 95) (* s 20) 0 0))

  (define bincomm (mk-bincomm 3))
  (define dircomm (mk-dircomm 3))

  (define (center-wrt a p)
    (cc-superimpose p (blank (pict-width a) 1)))
    
  (define together-arrows
    (hc-append
     (pict-width dircomm)
     downright-arrow
     downleft-arrow))
    
  (define straight-arrows
    (hc-append
     font-size
     (center-wrt scheme-angel-file down-arrow)
     (ghost (launder dircomm))
     (center-wrt scheme-angel-file down-arrow)))

  (define (label-left s p)
    (let ([t (text s `(bold . ,main-font) font-size)]
          [sep (/ font-size 2)])
      (inset (hc-append sep p t) 0 0 (- 0 sep (pict-width t)) 0))) 
  
  ;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  (define (angel-slides)
    (with-steps
        (angel devil mred angels angels-run bincomm multi-mred dircomm mredos)
      (slide/title
       (cond
         [(between? angel mred) "Programming in Heaven"]
         [(between? angels bincomm) "Multi-Programming"]
         [(between? multi-mred mredos) "Multi-Programming in Heaven"])  
       (cc-superimpose
        (vc-append
         (/ font-size 2)
         (hc-append
          font-size
          ((vafter angels) scheme-angel-file)
          (cc-superimpose
           ((vbetween-excl angel angels) scheme-angel-file)
           ((vafter dircomm) dircomm))
          ((vafter angels) scheme-angel-file))
         (cc-superimpose
          ((vbetween-excl mred angels) down-arrow)
          ((vbetween-excl angels-run multi-mred) straight-arrows)
          ((vafter multi-mred) together-arrows))
         (hc-append
          font-size
          ((vbetween-excl angels-run multi-mred) (center-wrt scheme-angel-file mred-logo))
          (cc-superimpose
           ((vbetween mred mred) (center-wrt dircomm (label-left "language run-time" mred-logo)))
           ((vafter mredos) (label-left "language as OS" mred-machine))
           ((vbetween-excl multi-mred mredos) (center-wrt dircomm mred-logo)))
          ((vbetween-excl angels-run multi-mred) (center-wrt scheme-angel-file mred-logo)))
         (cc-superimpose
          ((vbetween mred mred) down-arrow)
          ((vbetween-excl multi-mred mredos) down-arrow)
          ((vbetween-excl angels-run multi-mred) together-arrows))
         ((vbetween-excl devil mredos) devil-machine))
        ((vbetween bincomm bincomm) bincomm)))))
  
  ;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  (define (simpler-angel-slides)
    (with-steps (machine binary devil programs os mred)
      (slide/title
       "Machines and Processes"
       (vc-append
	font-size
	((vafter programs)
	 (hc-append
	  (pict-width dircomm)
	  scheme-angel-file
	  scheme-angel-file))
	((vafter os)
	 (vc-append
	  (/ font-size 2)
	  together-arrows
	  (hc-append
	   (cc-superimpose
	    ((vbefore mred) (center-wrt dircomm (label-left "OS" win-logo)))
	    ((vafter mred) (center-wrt dircomm (label-left "VM" mred-logo)))))
	  down-arrow))
	(cc-superimpose
	 ((vonly machine) (desktop-machine machine-size '()))
	 ((vonly binary) (desktop-machine machine-size '(binary)))
	 ((vafter devil) (desktop-machine machine-size '(devil)))))))))
  